@@ -386,11 +386,28 @@ const char *ast_get_cc_callback_macro(struct ast_cc_config_params *config);
  * \since 1.8
  * \brief Set the callback_macro name
  * \param config The configuration to set the callback_macro on
- * \param value The new callback macro  we want to change to
+ * \param value The new callback macro we want to change to
  * \retval void
  */
 void ast_set_cc_callback_macro(struct ast_cc_config_params *config, const char * const value);
 
+/*!
+ * \since 11
+ * \brief Get the name of the callback subroutine
+ * \param config The configuration to retrieve the callback_sub from
+ * \return The callback_sub name
+ */
+const char *ast_get_cc_callback_sub(struct ast_cc_config_params *config);
+
+/*!
+ * \since 11
+ * \brief Set the callback subroutine name
+ * \param config The configuration to set the callback_sub on
+ * \param value The new callback subroutine we want to change to
+ * \retval void
+ */
+void ast_set_cc_callback_sub(struct ast_cc_config_params *config, const char * const value);
+
 /* END CONFIGURATION FUNCTIONS */
 
 /* BEGIN AGENT/MONITOR REGISTRATION API */
@@ -858,6 +875,15 @@ struct ast_cc_agent {
 	char device_name[1];
 };
 
+enum ast_cc_agent_response_reason {
+	/*! CC request accepted */
+	AST_CC_AGENT_RESPONSE_SUCCESS,
+	/*! CC request not allowed at this time. Invalid state transition. */
+	AST_CC_AGENT_RESPONSE_FAILURE_INVALID,
+	/*! Too many CC requests in the system. */
+	AST_CC_AGENT_RESPONSE_FAILURE_TOO_MANY,
+};
+
 struct ast_cc_agent_callbacks {
 	/*!
 	 * \brief Type of agent the callbacks belong to.
@@ -920,19 +946,23 @@ struct ast_cc_agent_callbacks {
 	 */
 	int (*stop_offer_timer)(struct ast_cc_agent *agent);
 	/*!
-	 * \brief Acknowledge CC request.
+	 * \brief Respond to a CC request.
 	 *
 	 * \param agent CC core agent control.
+	 * \param reason CC request response status.
 	 *
 	 * \details
 	 * When the core receives knowledge that a called
 	 * party has accepted a CC request, it will call
-	 * this callback.
+	 * this callback. The core may also call this
+	 * if there is some error when attempting to process
+	 * the incoming CC request.
 	 *
-	 * The duty of this is to accept a CC request from
-	 * the caller by acknowledging receipt of that request.
+	 * The duty of this is to issue a propper response to a
+	 * CC request from the caller by acknowledging receipt
+	 * of that request or rejecting it.
 	 */
-	void (*ack)(struct ast_cc_agent *agent);
+	void (*respond)(struct ast_cc_agent *agent, enum ast_cc_agent_response_reason reason);
 	/*!
 	 * \brief Request the status of the agent's device.
 	 *
@@ -1335,8 +1365,8 @@ int ast_cc_monitor_count(const char * const name, const char * const type);
  * The code in the core will take care of making sure that the information gets passed
  * up the ladder correctly.
  *
- * \param core_id The core ID of the corresponding CC transaction
- * \param debug
+ * \par core_id The core ID of the corresponding CC transaction
+ * \par debug
  * \retval 0 Request successfully queued
  * \retval -1 Request could not be queued
  */
@@ -1452,13 +1482,15 @@ int ast_cc_agent_set_interfaces_chanvar(struct ast_channel *chan);
 /*!
  * \since 1.8
  * \brief Set the CC_INTERFACES channel variable for a channel using an
- * extension@context as a starting point
+ * \verbatim extension@context \endverbatim as a starting point
  *
  * \details
- * The CC_INTERFACES channel variable will have the interfaces that should be
- * called back for a specific PBX instance. This version of the function is used
- * mainly by chan_local, wherein we need to set CC_INTERFACES based on an extension
- * and context that appear in the middle of the tree of dialed interfaces
+ * The CC_INTERFACES channel variable will have the interfaces
+ * that should be called back for a specific PBX instance.  This
+ * version of the function is used mainly by local channels,
+ * wherein we need to set CC_INTERFACES based on an extension
+ * and context that appear in the middle of the tree of dialed
+ * interfaces.
  *
  * \note
  * This function will lock the channel as well as the list of monitors stored
@@ -1468,7 +1500,7 @@ int ast_cc_agent_set_interfaces_chanvar(struct ast_channel *chan);
  *
  * \param chan The channel to set the CC_INTERFACES variable on
  * \param extension The name of the extension for which we're setting the variable.
- * This should be in the form of "exten@context"
+ * This should be in the form of \verbatim exten@context \endverbatim
  */
 int ast_set_cc_interfaces_chanvar(struct ast_channel *chan, const char * const extension);
 
@@ -1524,7 +1556,7 @@ void ast_cc_busy_interface(struct ast_channel *inbound, struct ast_cc_config_par
  * ready, and then based on set flags, creates the proper frame type. For chan_dahdi, we
  * provide this function. It provides us the data we need, and we'll make its frame for it.
  *
- * \param chan A channel involved in the call. What we want is on a datastore on both incoming 
+ * \param chan A channel involved in the call. What we want is on a datastore on both incoming
  * and outgoing so either may be provided
  * \param cc_params The CC configuration parameters for the outbound target
  * \param monitor_type The type of monitor to use when CC is requested
@@ -1535,7 +1567,7 @@ void ast_cc_busy_interface(struct ast_channel *inbound, struct ast_cc_config_par
  * data has been allocated, then this parameter should contain a pointer to that data. If using a generic
  * monitor, this parameter should remain NULL. Note that if this function should fail at some point,
  * it is the responsibility of the caller to free the private data upon return.
- * \param[out] frame The frame we will be returning to the caller. It is vital that ast_frame_free be 
+ * \param[out] frame The frame we will be returning to the caller. It is vital that ast_frame_free be
  * called on this frame since the payload will be allocated on the heap.
  * \retval -1 Failure. At some point there was a failure. Do not attempt to use the frame in this case.
  * \retval 0 Success
